//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef CREATE_SCENARIO_H
#define CREATE_SCENARIO_H

#include <string>
#include <sigc++/sigc++.h>
#include <SDL.h>

#include "player.h"
#include "playerlist.h"
#include "stacklist.h"
#include "citylist.h"
#include "ruinlist.h"
#include "templelist.h"
#include "armysetlist.h"
#include "MapGenerator.h"
#include "GamePreferencesDialog.h"
#include "Event.h"

/** \brief Creates and dumps (i.e. saves) a scenario.
  * 
  * The purpose of this class is to create a new scenario which can then be
  * "loaded" by the game scenario class. The advantage is that the scenario
  * creation can be concentrated in one specific class instead of being scattered
  * over different classes and all objects (city, player, stack...) only need
  * one "loading" constructor. The disavantage is that all significant changes
  * e.g. to the player class have to be reflected here, too.
  */

class CreateScenario : public SigC::Object
{
    public:
        //! This represents the class of the map (hills, islands etc.).
        enum MapType {NORMAL};
    
        /** The Constructor
          * 
          * @param uncle    the progress bar which displays the advance
          */
        CreateScenario(PG_Widget* uncle);
        ~CreateScenario();

        // setters

        //! Set the type of the map to be created
        void setMaptype(MapType type);

        //! Set the terrain distribution; differences to 100% are grass
        void setPercentages(int pgrass, int pwater, int pforest, int pswamp,
                            int phills, int pmountains);

        //! Set the tileset of the map
        void setMapTiles(std::string tilesname);

        //! Set the number of cities on the map
        void setNoCities(int number);

        //! Set the number of ruins on the map
        void setNoRuins(int number);

        //! Set the number of temples
        void setNoTemples(int number);

        //! Set the size of the map (always a square)
        void setSize(int size);

        /** Set the turn mode
          * 
          * @param turnmode     if set to true, several actions (healing armies
          *                     and producng armies, respectively) take part at
          *                     the beginning of the correpsonding player's turn,
          *                     else they are done when all players have
          *                     finished their round.
          */
        void setTurnmode(bool turnmode) {d_turnmode=turnmode;}
      
        /** Add a player to the scenario
          * 
          * @param name     the name of the player
          * @param armyset  the name of the player's armyset
          * @param color    the color of the player
          * @param type     the type of the player (see class player for more info)
          */
        void addPlayer(std::string name, std::string armyset, SDL_Color color,
                        Player::TYPE type);

        
        //! Get the number of players already added
        int getNoPlayers();

        //! Get the n-th player
        Player* getPlayer(int number);

        //! Get the type of the map
        MapType getMaptype();

        //! Get the number of cities of the map
        int getNoCities();

        //! Get the number of ruins on the map
        int getNoRuins();

        //! Get the number of temples
        int getNoTemples();

        //! Get the tileset to be used for the map
        std::string getMapTiles();
        
        
        /** Creates a map
          * 
          * Calling this function will initiate the creation process itself. The
          * result is a saved map with several distributed players. The created
          * map can be either saved or used further (all lists etc. have already
          * been filled)
          */
        bool create();

        /** Dumps (Saves) the map
          * 
          * @param filename     the full name of the save file
          * 
          * This will do the same as GameScenario::save() does.
          */
        bool dump(std::string filename);

        //this signal will frequently be called whenever a step is completed
        SigC::Signal1<void, int> sstepfinished;

    private:
        //! Creates the map and distributes cities, temples and ruins
        bool createMap();

        //! Distributes the players over the map
        bool distributePlayers();

        //! Setup city names and production
        bool setupCities();

        //! Set up ruin keepers
        bool setupRuins();

        //! Do some setup concerning the players (grant each a hero etc.)
        bool setupPlayers();

        //! Loads a list of possible city names
        bool loadCitynames();

        //! Sets up some basic events to satisfy victory conditions
        bool setupEvents();


        //data
        //for map creation
        MapGenerator* d_generator;
        std::string d_tilesname;
        int d_size;
        bool d_turnmode;
        std::list<Event*> d_events;

        PG_Widget* d_uncle;

        //the namelists
        std::vector<std::string> d_citynames;
};

#endif  //CREATE_SCENARIO_H
