//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef TILE_H
#define TILE_H

#include <string>
#include <SDL.h>

/** Class which describes the terrain
  * 
  * Many tiles are put together to form a tileset. Thus, a tile describes a
  * single terrain type. It keeps the name, movement points, type (grass, water
  * etc.) as well as the images together.
  */

class Tile
{
    public:
        // CREATORS
        Tile();
        ~Tile();

        //! Set the name (e.g. forest, water) of the tile
        void setName(std::string name){d_name = name;}

        //! Set the number of MP needd to cross the tile
        void setMoves(int moves){d_moves = moves;}

        //! Set the defense bonus for the defender (10 == defense doubled)
        void setDefense(int defense){d_defense = defense;}

        //! Set the color which the terrain has on the smallmap
        void setColor(SDL_Color& color){d_color = color;}

        //! Set the images of the terrain. See MapRenderer how they are used.
        void setSurface(int t, SDL_Surface* surface){d_surface[t] = surface;}

        //! Set the type (grass, mountain, hills...) of the tile
        void setType(unsigned int type) {d_type = type;}

        //! Prints some debug information
        void printDebugInfo();

        
        //! Get a surface . See MapRenderer how they are used.
        SDL_Surface* getSurface(int t);

        //! Get the name of the terrain
        std::string getName(){return d_name;}

        //! Get the number of movement points needed to cross this tile
        int getMoves(){return d_moves;}

        //! Get the defense bonus (e.g. 5 means that defense goes up by 50%)
        int getDefense(){return d_defense;}

        //! Get the color associated with this tile for the smallmap
        SDL_Color getColor(){return d_color;}

        //! Get the type (grass, hill,...) of this tile type
        unsigned int getType(){return d_type;}
                
    private:
        // DATA
        SDL_Surface* d_surface[8*4];      // base tiles * 4 = corners
        std::string d_name;                // name    
        int d_moves;                    // moves needed to walk over maptile
        int d_defense;                    // defense bonus
        SDL_Color d_color;                // color shown in the smallmap
        unsigned int d_type;
};

#endif // TILE_H

// End of file
