//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef ARMY_H
#define ARMY_H

#include <string>
#include <vector>
#include <sigc++/sigc++.h>
#include "armyset_army.h"
#include "xmlhelper.h"

class Armyset;
class Player;

/** Class which describes an army.
  * 
  * This class is used to hold the information about a single unit in the actual
  * game.
  */

class Army : public Armyset_army, public SigC::Object
{
    public:
        /** Default constructor
          * 
          * @param armyset      the armyset which contains the type of the army
          * @param type         the type of the unit (== index in the armyset)
          * @param owner        the player owning this army
          */
        Army(Armyset* armyset, int type, Player* owner);

        //! Copy constructor
        Army(Army& a);

        /** Loading constructor. See XML_Helper for more details.
          *
          * @param helper       the XML_Helper instance of the savegame
          */
        Army(XML_Helper* helper);
        
        ~Army();

        
        //! Set the player owning this army
        void setOwner(Player* owner) {d_owner = owner;}

        //! Set the number of hitpoints of this army
        void setHP(int hp) {d_hp = hp;}

        /** Set the grouping information of this army.
          * 
          * Background: When you select a stack on the map, you can toggle the
          * armies which will be moved when you click somewhere else on the map
          * then. Grouped==true means that this army will be among the selected
          * armies.
          */
        void setGrouped(bool grouped){d_grouped = grouped;}

        
        //! Get the unique id of this army
        unsigned int getId() const {return d_id;}

        //! Get the type of this army
        int getType() const {return d_type;}

        //! Get the player owning this army
        Player* getOwner() const {return d_owner;}

        //! Get the image of the army. Internally, this refers to GraphicsCache.
        virtual SDL_Surface* getPixmap();

        //! Get the current number of hitpoints (in opposite to getMaxHp)
        int getHP() const {return d_hp;}

        //! Get the current number of moves (in opposite to getMaxMoves)
        int getMoves() const {return d_moves;}

        /** Get the actual strength of the unit. This may differ from the
          * raw strength e.g. because of enchantments or items (in the case
          * of heroes). To get the raw strength, use getOrigStrength().
          */
        virtual int getStrength();

        //! Same for the defense; use getOrigDefense to get the basic value.
        virtual int getDefense();

        //! Same for the vitality. Use getOrigVitality to get the base value.
        virtual int getVitality();

        //! Same for the maximum hitpoints. Use getOrigMaxHp for the base value.
        virtual int getMaxHp();

        //! Same for the maximum MP. use getOrigMaxMoves for the base value.
        virtual int getMaxMoves();

        //! Same for the move bonus. Use getOrigMoveBonus for the base value
        virtual unsigned int getMoveBonus();

        //! Same for the army bonus. Use getOrigArmyBonus for the base value
        virtual unsigned int getArmyBonus();

        
        //! Get the number of experience points of the unit
        double getXP() const {return d_xp;}

        //! Get the level of the army
        int getLevel() const {return d_level;}

        //! Returns whether army is blessed (may only happen once :-))
        bool isBlessed() const {return d_blessed;}

        //! Returns grouping information
        bool isGrouped() const {return d_grouped;}

        //! Returns whether the army is a hero
        virtual bool isHero() {return false;}

        
        /** Heal the unit
          * 
          * @param hp       hitpoints to hal. Set to zero for "natural" healing.
          */
        void heal(int hp = 0);

        /** Damage this unit
          * 
          * @param damageDone       how much damage the unit suffers
          * @return true if the unit has died, else otherwise
          */
        bool damage(int damageDone);

        /** Reduce the number of moves
          * 
          * @param moves            how many movement points have been spent
          */
        void decrementMoves(int moves);

        //! Resets the number of movement points to maximum
        void resetMoves();

        //! Blesses the unit (currently: strength+1)
        void bless();

        //! Increases the XP of the unit
        void gainXp(double n);

        //! Checks whether unit can advance a level
        bool canGainLevel();

        //! Has the unit advance a level
        void gainLevel();

        bool * getMedalBonuses() {return (bool*)&d_medal_bonus;}
        bool getMedalBonus(unsigned int index) {return d_medal_bonus[index];}
        void setMedalBonus(unsigned int index, bool value) {d_medal_bonus[index]=value;}

        unsigned int getBattlesNumber() {return d_battles_number;}
        void setBattlesNumber(unsigned int value) {d_battles_number=value;}
         
        unsigned int getNumberHasHit() {return d_number_hashit;}
        void setNumberHasHit(unsigned int value) {d_number_hashit=value;}

        unsigned int getNumberHasBeenHit() {return d_number_hasbeenhit;}
        void setNumberHasBeenHit(unsigned int value) {d_number_hasbeenhit=value;}

        unsigned int getNumberAttacks() {return d_number_attacks;}
        void setNumberAttacks(unsigned int value) {d_number_attacks=value;}

        void printAllDebugInfo();

        //! Saves the unit information (see XML_Helper for further info)
        virtual bool save(XML_Helper* helper);       
        
        //! This signal is raised when the army dies.
        SigC::Signal1<void, Army*> sdying;
        
    protected:
        //! Generic function for saving the army data. Useful for the hero class,
        //  which doesn't need to repeat the save code.
        bool saveData(XML_Helper* helper);
        
        Armyset* d_armyset;
        Player* d_owner;
        unsigned int d_id;
        int d_type;
        int d_hp;
        int d_moves;
        double d_xp;            // experience points
        int d_level;
        bool d_blessed;            // if army is blessed then strength is +1
        bool d_grouped;

        //! Medal bonuses : we have 3 medals
        //!  0 _ medal for being extreme mercyless (gives an extra +1 to strength)
        //!      a unit gets this medal if in a combat it scores more that 90% of hits          
        //!  1 _ medal for being very good in defense (gives an extra +1 to defense)      
        //!      a unit gets this medal if in a combat is never hit          
        //!  2 _ medal for being very good in combat  (gives an extra +4 to Hit Points)       
        //!      a unit gets this medal if it survives a lot of battles (for now 10 battles)
        bool d_medal_bonus[3];

        //! number of fought battles 
	unsigned int d_battles_number;
        //! number of hits per battle
	unsigned int d_number_hashit;
        //! number of times the army has been hit in a battle
	unsigned int d_number_hasbeenhit;
        //! number of attacks per battle
	unsigned int d_number_attacks;
};

#endif // ARMY_H
