//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef ARMYSET_ARMY_H
#define ARMYSET_ARMY_H

#include <SDL.h>
#include <string>

/** Description of  a single army type
  * 
  * This class is the atom of every armyset. It contains all data related to
  * a single army type of an armyset, such as strength, defense, movement points
  * and so on.
  */

class Armyset_army
{
    public:
        Armyset_army();
        Armyset_army(const Armyset_army& a);
        virtual ~Armyset_army();

        //! Copy the values of another army type
        void copyVals(const Armyset_army* a);

        //! Set the name of the unit type
        void setName(std::string name){d_name = name;}

        //! Set the basic image of the unit type
        void setPixmap(SDL_Surface* pixmap){d_pixmap = pixmap;}

        //! Set the mask of the unit type (for player colors)
        void setMask(SDL_Surface* mask){d_mask = mask;}

        //! Set the maximum number of hitpoints of the unit type
        void setOrigMaxHp(int max_hp){d_max_hp = max_hp;}

        //! Set how much XP this unit is worth when killed
        void setXpReward(double xp_value){d_xp_value = xp_value;}

        //! Set the strength of the unit type
        void setOrigStrength(int max_strength){d_strength = max_strength;}
        
        //! Set the defense
        void setOrigDefense(int defense){d_defense = defense;}

        //! Set the vitality (currently unused)
        void setOrigVitality(int vitality){d_vitality = vitality;}

        //! Set the maximum number of movement points
        void setOrigMaxMoves(int max_moves){d_max_moves = max_moves;}

        //! Set how much gold this unit needs per turn
        void setUpkeep(int upkeep){d_upkeep = upkeep;}
        
        //! Set how many turns this unit type needs to be produced
        void setProduction(int production){d_production = production;}

        //! Set the amount of gold needed to buy a city this production
        void setProductionCost(int production_cost){d_production_cost = production_cost;}

        //! Set which terrain the unit can easily cross
        void setOrigMoveBonus(unsigned int move_bonus){d_move_bonus = move_bonus;}

        //! Set army boni like hero, ship, ... (see defs.h for the macros)
        void setOrigArmyBonus(unsigned int army_bonus){d_army_bonus = army_bonus;}

        //! Prints debug information
        void printDebugInfo();
        
        
        //! Returns the name of the unit type
        std::string getName() const {return d_name;}

        //! Returns the pixmap (Attention: Use it read-only!!!)
        virtual SDL_Surface* getPixmap() const {return d_pixmap;}

        //! Returns the mask (read-only!!) for player colors
        SDL_Surface* getMask() const {return d_mask;}

        //! Returns the strength of the unit type
        int getOrigStrength() const {return d_strength;}

        //! Returns the maximum number of movement points
        int getOrigMaxMoves() const {return d_max_moves;}

        //! Returns how much gold this unit type needs per turn
        int getUpkeep() const {return d_upkeep;}

        //! returns how many turns this unit type needs to be produced
        int getProduction() const {return d_production;}

        //! Returns the defense value of this unit type
        int getOrigDefense() const {return d_defense;}

        //! Returns the vitality of this unit type (currently unused)
        int getOrigVitality() const {return d_vitality;}

        //! Returns how much gold setting up the production costs
        int getProductionCost() const {return d_production_cost;}

        //! Returns the maximum number of hitpoints of this unit type
        int getOrigMaxHp() const {return d_max_hp;}

        //! Returns how much XP killing a unit of this type is worth
        double getXpReward() const {return d_xp_value;}

        //! Returns the movement bonus (see defs.h)
        unsigned int getOrigMoveBonus() const {return d_move_bonus;}

        //! Returns the army bonus (see defs.h)
        unsigned int getOrigArmyBonus() const {return d_army_bonus;}

    protected:
        std::string d_name;
        SDL_Surface* d_pixmap;
        SDL_Surface* d_mask;
        int d_production;
        int d_production_cost;
        int d_upkeep;
        int d_strength;
        int d_defense;
        int d_max_hp;
        int d_max_moves;
        int d_vitality;
        double d_xp_value;
        unsigned int d_move_bonus;
        unsigned int d_army_bonus;
};

#endif // ARMYSET_ARMY_H

// End of file
