//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "fight.h"
#include <assert.h>
#include <algorithm>
#include <stdlib.h>     // for random numbers
#include <math.h>       // for has_hit()
#include "army.h"
#include "stacklist.h"
#include "citylist.h"
#include "city.h"
#include "stack.h"
#include "defs.h"

using namespace std;

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
#define debug(x)

Fight::Fight(Maptile* maptile, Stack* attacker, Stack* defender, bool def_bonus)
    :d_mapTile(maptile)
{
    assert(attacker->size() != 0);
    assert(defender->size() != 0);
    
    for (Stack::iterator it=attacker->begin(); it != attacker->end(); it++)
        d_attacker.push_back(*it);
    
    for (Stack::iterator it=defender->begin(); it != defender->end(); it++)
        d_defender.push_back(*it);
    
    d_extra_damage = 0;
    d_add_defense = 1;
    d_open_fight = def_bonus;
    // used in fight_armies, calculateDefenseBonus later

    d_pos = defender->getPos();
    if (def_bonus)
    {
        calculateDefenseBonus(defender);    
    }
    d_turn = 0;
    checkHero();
}

Fight::Fight(Maptile* maptile, vector<Stack*>& attacker,
        vector<Stack*>& defender, bool def_bonus)
    :d_mapTile(maptile)
{
    Stack::iterator it;

    for (unsigned int i = 0; i < attacker.size(); i++)
        for (it=attacker[i]->begin(); it!=attacker[i]->end(); it++)
            d_attacker.push_back(*it);
    assert(!d_attacker.empty());
    
    for (unsigned int i = 0; i < defender.size(); i++)
        for(it=defender[i]->begin(); it!=defender[i]->end(); it++)
            d_defender.push_back(*it);
    assert(!d_defender.empty());

    d_extra_damage = 0;
    d_add_defense = 1;
    d_open_fight = def_bonus;
    // used in fight_armies, calculateDefenseBonus later

    d_pos = defender[0]->getPos();
    if (def_bonus)
    {
        calculateDefenseBonus(defender[0]);
    }
    d_turn = 0;
    checkHero();
}

Fight::~Fight()
{
}

// calculate the general defense bonus for the defenders
void Fight::calculateDefenseBonus(Stack* stack)
{
    d_pos = stack->getPos();    // used in fight_armies later

    //TBD: If the city is razed, what do we add?
    if (d_mapTile->getBuilding() == Maptile::CITY )
    {
        City* city = Citylist::getInstance()->getObjectAt(stack->getPos());
        d_add_defense = 1 + (city->getDefenseLevel() * 0.2);
        d_open_fight = false;
    }
    else
    {
        d_add_defense = d_mapTile->getDefense() / 10.0 + 1;
        d_open_fight = true;
        
        if (d_mapTile->getBuilding() == Maptile::RUIN )
            d_add_defense += 0.1;

        if (d_mapTile->getMoves() > 4 )
    	    d_open_fight = false; 
	   
    }
}


bool Fight::newRound()
{
    d_round_finished = false;
    d_attacker_finished = false;
    d_active = d_attacker.begin();
    d_turn++;

    return (d_turn < MAX_ROUNDS);
}


Fight::RESULT Fight::nextFight()
{
    debug("nextFight()");
    // if there haven't been any defender

    if (!d_attacker_finished)
    {
        int random = rand() % d_defender.size();
        checkHero();

        if ((fightArmies(*d_active, d_defender[random], true))
            && (d_defender[random]->getHP() <= 0))
        {
            //defender army has been killed
            //this is for efficiency, deques are made to erase the
            //first/last item very fast
            d_defender[random] = d_defender[0];
            d_defender.erase(d_defender.begin());

            if (d_defender.empty())
                return ATTACKER_WON;
        }

        if (++d_active == d_attacker.end())
        {
            d_active = d_defender.begin();
            d_attacker_finished = true;
        }

        return NONE;
    }

    //else defender's turn
    int random =rand() % d_attacker.size();
    checkHero();
    if ((fightArmies(*d_active, d_attacker[random], false))
        && (d_attacker[random]->getHP() <= 0))
    {
        //attacker army has been killed
        d_attacker[random] = d_attacker[0];
        d_attacker.erase(d_attacker.begin());

        if (d_attacker.empty())
            return DEFENDER_WON;
    }

    if (++d_active == d_defender.end())
    {
        d_round_finished = true;
    }

    return NONE;
}


bool Fight::fightArmies(Army* culprit, Army* victim, bool attack)
{
    debug("fightArmies()");

    unsigned int culprit_bonus = culprit->getArmyBonus();
    unsigned int victim_bonus = victim->getArmyBonus();


    float strength = culprit->getStrength();
    float defense = victim->getDefense();

    //add general defense bonus to victim
    if (attack)
    {
        defense *= d_add_defense;
    }

    //now comes the evaluation of several special army abilities as defined
    //in defs.h

    //if army is cavalry and open terrain and enemy not anticavalry, 
    //add +1 to strength
    //TBD: maptile type is only compared with MAPTILE_NONE, which could be  
    //a problem with other maptilesets than the standard one. Fix it!!
    //(don't know how to do it exactly)
    if (d_open_fight && (culprit_bonus & ARMY_CAVALRY) && attack
            && (d_mapTile->getMaptileType() == MAPTILE_NONE)
            && !(victim_bonus & ARMY_ANTICAVALRY))
        strength++;

    //if hero is on attackers side, increase strength of the attacking army
    //(unless it is a hero itself)
    if (d_atthero && attack && !(culprit->getArmyBonus() & ARMY_HERO))
        strength++;

    //a hero on the defenders side increases the strength of the retaliating
    //defender
    if (d_defhero && !attack && !(culprit->getArmyBonus() & ARMY_HERO))
        strength++;
    
    //if anticavalry unit and enemy is cavalry, double strength
    if ((culprit_bonus & ARMY_ANTICAVALRY) && (victim_bonus & ARMY_CAVALRY))
        strength *= 2;

    culprit->setNumberAttacks(culprit->getNumberAttacks()+1);
    victim->setNumberAttacks(victim->getNumberAttacks()+1);

    //unit can do instant kills with a chance of ... 1%
    if (culprit_bonus & ARMY_CRITICAL)
        if ((rand() % 100) == 0)
        {
            culprit->setNumberHasHit(culprit->getNumberHasHit()+1);
            victim->damage(victim->getHP());
            return true;
        }

    if (hasHit(strength, defense))
    {
        culprit->setNumberHasHit(culprit->getNumberHasHit()+1);
	victim->setNumberHasBeenHit(victim->getNumberHasBeenHit()+1);
        victim->damage(1 + d_extra_damage);
	d_extra_damage = 0; /* is this necessary? */
        return true;
    }

    return false;
}

// The algorithm used here is as follows. We take a function
// f(x) = 50 * 0.75^(x)
// with x being the difference between attack strength and defense value.
// If the attack strength is smaller, the propability of the attacker hitting
// the defender is f(x), else it is (1 - f(x)). This has the advantage that
// the propability of a weak unit to hit a heavily armoured one steadily 
// goes towards zero with increasing inequality, but non-linear.

// There is always a limit of at least 5% /at most 95% to hit a unit (to
// give really weak units at least the chance to hit a stronger one before 
// being killed)

bool Fight::hasHit(float att_strength, float def_strength)
{
    int diff = abs ((int) (att_strength - def_strength));
    int chance = (int) (50 *(pow (0.75, diff)));
    int roll = rand() % 100;
    if (chance < 5) chance = 5;
    else if (chance > 95) chance = 95;

    if (att_strength > def_strength) chance = 100 - chance;

    if (roll <= chance)
    {
	if ((chance - roll) > 50) d_extra_damage += 1;
	if ((chance - roll) > 80) d_extra_damage += 1;
	return true;
    } else { 
	return false;
    }
}

//check and mark if there is a hero in the stack

void Fight::checkHero()
{
    //Search the attackers for a hero...
    for (deque<Army*>::iterator it = d_attacker.begin();
        it != d_attacker.end(); it++)
    {
        if ((*it)->getArmyBonus() & ARMY_HERO)
        {
            d_atthero = true;
            break;
        }
    }

    for (deque<Army*>::iterator it = d_defender.begin();
        it != d_defender.end(); it++)
    {
        if ((*it)->getArmyBonus() & ARMY_HERO)
        {
            d_defhero = true;
            break;
        }
    }
}

// End of file
