//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#include <stdlib.h>
#include <fstream>

#include "hero.h"
#include "armyset.h"
#include "File.h"


std::vector<std::string> Hero::s_namelist;

Hero::Hero(Armyset* armyset, int type, std::string name, Player* owner)
    :Army(armyset, type, owner)
{
    d_name = name;

    if (s_namelist.empty())
        loadNames();

    if (name == "")  //set a random name
    {
        int entry = rand() % s_namelist.size();
        d_name = s_namelist[entry];
    }
}

Hero::Hero(Armyset* armyset, std::string name, Player* owner)
    :Army(armyset, rand()%2, owner)
{
    d_name = name;
    
    if (s_namelist.empty())
        loadNames();

    if (name == "")  //set a random name
    {
        int entry = rand() % s_namelist.size();
        d_name = s_namelist[entry];
    }
}

Hero::Hero(XML_Helper* helper)
    :Army(helper)
{
    helper->getData(d_name, "name");
    helper->registerTag("backpack", SigC::slot(*this, &Hero::loadItems));
    helper->registerTag("equipment", SigC::slot(*this, &Hero::loadItems));
    helper->registerTag("item", SigC::slot(*this, &Hero::loadItems));
}

Hero::Hero(Hero& h)
    :Army(h)
{
    d_name = h.d_name;
    std::list<Item*>::iterator it;

    // copy the backpack of the other hero
    for (it = h.d_backpack.begin(); it != h.d_backpack.end(); it++)
    {
        Item* item = new Item(**it);
        d_backpack.push_back(item);
    }

    // copy the equipment of the other hero
    for (it = h.d_equipment.begin(); it != h.d_equipment.end(); it++)
    {
        Item* item = new Item(**it);
        d_equipment.push_back(item);
    }
}

Hero::~Hero()
{
    //clear the backpack and the equipment
    while (!d_backpack.empty())
    {
        delete (*d_backpack.begin());
        d_backpack.erase(d_backpack.begin());
    }

    while (!d_equipment.empty())
    {
        delete (*d_equipment.begin());
        d_equipment.erase(d_equipment.begin());
    }
}

bool Hero::save(XML_Helper* helper)
{
    bool retval = true;
    std::list<Item*>::iterator it;

    retval &= helper->openTag("hero");
    retval &= helper->saveData("name", d_name);

    retval &= saveData(helper);

    // Now save the equipment and the backpack
    retval &= helper->openTag("backpack");
    for (it = d_backpack.begin(); it != d_backpack.end(); it++)
        retval &= (*it)->save(helper);
    retval &= helper->closeTag();

    retval &= helper->openTag("equipment");
    for (it = d_equipment.begin(); it != d_equipment.end(); it++)
        retval &= (*it)->save(helper);
    retval &= helper->closeTag();
    
    retval &= helper->closeTag();

    return retval;
}

bool Hero::loadItems(std::string tag, XML_Helper* helper)
{
    // determines whether an item is put in the backpack or to the equipment
    static bool equipment = false;

    if (tag == "backpack")
    {
        equipment = false;
        return true;
    }

    if (tag == "equipment")
    {
        equipment = true;
        return true;
    }

    if (tag == "item")
    {
        Item* item = new Item(helper);
        if (equipment)
            d_equipment.push_back(item);
        else
            d_backpack.push_back(item);
    }
    
    return true;
}

void Hero::loadNames()
{
    //Load hero names from a name file. This could certainly be improved
    //(e.g. separating male/female hero names), but should work well
    //enough for now.
    ifstream name_file(File::getMiscFile("heronames").c_str());
    if (name_file)
    {
        int entries;
        std::string curname;
        
        name_file >> entries;

        for (int i = 0; i < entries; i++)
        {
            name_file >> curname;
            s_namelist.push_back(curname);
        }

        name_file.close();
    }
}

int Hero::getStrength()
{
    int strength = d_strength;

    // Add item boni
    std::list<Item*>::iterator it;
    for (it = d_equipment.begin(); it != d_equipment.end(); it++)
        if ((*it)->getBonus(Item::STRENGTH))
            strength += (*it)->getValue(Item::STRENGTH);
    
    return strength;
}

int Hero::getDefense()
{
    int defense = d_defense;

    //Add item boni
    std::list<Item*>::iterator it;
    for (it = d_equipment.begin(); it != d_equipment.end(); it++)
        if ((*it)->getBonus(Item::DEFENSE))
            defense += (*it)->getValue(Item::DEFENSE);

    return defense;
}

int Hero::getVitality()
{
    int vitality = d_vitality;

    //Add item boni
    std::list<Item*>::iterator it;
    for (it = d_equipment.begin(); it != d_equipment.end(); it++)
        if ((*it)->getBonus(Item::VITALITY))
            vitality += (*it)->getValue(Item::VITALITY);

    return vitality;
}

int Hero::getMaxHP()
{
    int hp = d_max_hp;

    //Add item boni
    std::list<Item*>::iterator it;
    for (it = d_equipment.begin(); it != d_equipment.end(); it++)
        if ((*it)->getBonus(Item::HP))
            hp += (*it)->getValue(Item::HP);

    return hp;
}

int Hero::getMaxMoves()
{
    int moves = d_max_moves;

    //Add item boni
    std::list<Item*>::iterator it;
    for (it = d_equipment.begin(); it != d_equipment.end(); it++)
        if ((*it)->getBonus(Item::MOVES))
            moves += (*it)->getValue(Item::MOVES);
    
    return moves;
}

unsigned int Hero::getMoveBonus()
{
    int mbonus = d_move_bonus;

    //Add item boni
    std::list<Item*>::iterator it;
    for (it = d_equipment.begin(); it != d_equipment.end(); it++)
        if ((*it)->getBonus(Item::MOVE_BONUS))
            mbonus |= (*it)->getValue(Item::MOVE_BONUS);
    
    return mbonus;
}

unsigned int Hero::getArmyBonus()
{
    int abonus = d_army_bonus;

    //Add item boni
    std::list<Item*>::iterator it;
    for (it = d_equipment.begin(); it != d_equipment.end(); it++)
        if ((*it)->getBonus(Item::ARMY_BONUS))
            abonus |= (*it)->getValue(Item::ARMY_BONUS);
    
    return abonus;
}

bool Hero::addToBackpack(Item* item, int position)
{
    std::list<Item*>::iterator it = d_backpack.begin();
    for (; position > 0; position--, it++);
    
    d_backpack.insert(it, item);
    return true;
}

bool Hero::addToEquipment(Item* item)
{
    //First, check if it is allowed to take the item at all
    Item::TYPE newtype = item->getType();

    std::list<Item*>::iterator it;
    for (it = d_equipment.begin(); it != d_equipment.end(); it++)
        if ((*it)->getType() == newtype)
            return false;
    
    //! OK, we don't have such an item already, so equip it
    d_equipment.push_back(item);
    return true;
}

bool Hero::removeFromBackpack(Item* item)
{
    std::list<Item*>::iterator it;
    for (it = d_backpack.begin(); it != d_backpack.end(); it++)
        if ((*it) == item)
        {
            d_backpack.erase(it);
            return true;
        }

    return false;
}

bool Hero::removeFromEquipment(Item* item)
{
    std::list<Item*>::iterator it;
    for (it = d_equipment.begin(); it != d_equipment.end(); it++)
        if ((*it) == item)
        {
            d_equipment.erase(it);
            return true;
        }

    return false;
}
