//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef HERO_H
#define HERO_H

#include <string>
#include <vector>
#include <list>

#include "army.h"
#include "Item.h"

/** A hero is a special army, as heroes are rather special, they have names, an
  * inventory...theoretically.
  *
  * The current hero implementation only gives heros a name
  */

class Hero : public Army
{
    public:
        /** Standard constructor, the male hero has to be the first, the female
          * the second army type in the armyset.
          * 
          * @param armyset          the armyset of the hero
          * @param type             the index of the army type in the armyset
          * @param name             the name of the hero
          * @param owner            the owning player
          */
        Hero(Armyset* armyset, int type, std::string name, Player* owner);

        /** Constructor which chooses the gender on a random basis.
          * 
          * @param armyset          the armyset of the hero
          * @param name             the name of the hero
          * @param owner            the owning player of the hero
          */
        Hero(Armyset* armyset, std::string name, Player* owner);

        /** Loading constructor. See XML_Helper as well.
          * 
          * @param helper           the XML_Helper instance of the savefile
          */
        Hero(XML_Helper* helper);

        //! Copy constructor
        Hero(Hero& h);
        ~Hero();

        //! Saves the hero data
        bool save(XML_Helper* helper);

        bool isHero() {return true;}

        
        //! Returns the actual strength of the hero (modified e.g. by items)
        int getStrength();

        //! Similarily, returns the actual defense of the hero
        int getDefense();

        //! Returns the actual vitality of the hero
        int getVitality();

        //! Returns the actual number of maximum hitpoints
        int getMaxHP();

        //! Return the actual maximum number of movement points
        int getMaxMoves();

        //! Returns the actual move bonus (modified by e.g. items)
        unsigned int getMoveBonus();

        //! Returns the actual army bonus (modified by e.g. items)
        unsigned int getArmyBonus();
        

        //! Add an item to the backpack of the hero. Returns true on success
        bool addToBackpack(Item* item, int position);

        //! Add an item to the list of equipped items. Returns true on success
        bool addToEquipment(Item* item);

        //! Remove an item from the backpack of the hero (don't delete it!)
        //! Returns true on success.
        bool removeFromBackpack(Item* item);

        //! Remove item from list of equipped items (don't delete it!)
        //! Returns true on success.
        bool removeFromEquipment(Item* item);

        //! Returns the equipment of the hero
        std::list<Item*> getEquipment() {return d_equipment;}

        //! Returns the backpack of the hero
        std::list<Item*> getBackpack() {return d_backpack;}


        //! Callback needed during loading of the hero (see xmlhelper.h)
        bool loadItems(std::string tag, XML_Helper* helper);
        
    private:
        //! Loads the file with hero names from which random names are chosen
        void loadNames();
        
        static std::vector<std::string> s_namelist;

        std::list<Item*> d_equipment;
        std::list<Item*> d_backpack;
};

#endif //HERO_H
