//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdlib.h>
#include <sstream>
#include "path.h"
#include "army.h"
#include "GameMap.h"
#include "citylist.h"
#include "city.h"
#include "stacklist.h"
#include "defs.h"
#include "stack.h"

using namespace std;

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl<<flush;}
#define debug(x)

Path::Path()
{
}

Path::Path(XML_Helper* helper)
{
    int i;
    std::istringstream sx, sy;
    std::string s;

    helper->getData(i, "size");

    helper->getData(s, "x");
    sx.str(s);
    helper->getData(s, "y");
    sy.str(s);

    for (; i > 0; i--)
    {
        PG_Point *p = new PG_Point;

        sx >> p->x;
        sy >> p->y;
        // d_dest is initialised to the last point in the path
        d_dest.x = p->x;
        d_dest.y = p->y;
        push_back(p);
    }
}

Path::~Path()
{
    fl_clear();
}

//checks whether path is still valid. This function detects obstacles in the
//path and relies on calculate_path to find a way around them
bool Path::checkPath(Stack* s)
{
    debug("check_path() " << d_dest.x << "," << d_dest.y);
    d_unit = s;
    bool blocked = false;
    for (iterator it = begin(); it != end(); it++)
    {
        if (isBlocked((*it)->x, (*it)->y, d_dest.x, d_dest.y))
        {
            blocked = true;
            break;
        }
    }
    if (blocked)
    {
        debug("recalculating blocked path dest = " << d_dest.x << "," << d_dest.y)
        fl_clear();
        blocked = calculate(s, d_dest);
    }
    return !blocked;
}

bool Path::canMoveThere(Stack* s, PG_Point *_dest)
{
    calculateStackMovementBonuses();
    PG_Point pos=s->getPos();
    return  (!isBlocked(pos.x,pos.y, _dest->x, _dest->y));
}

//calculate the path for the stack. Almost dummy function.
bool Path::calculate (Stack* s, PG_Point dest)
{
    PG_Point start = s->getPos();
    debug("path from "<<start.x<<","<<start.y<<" to "<<dest.x<<","<<dest.y)
    fl_clear();
    d_dest = dest;
    d_unit = s;
    calculateStackMovementBonuses();
    if (isBlocked(d_dest.x, d_dest.y, d_dest.x, d_dest.y)) return false;
    int size = GameMap::getSize();
    int length = size * size;
    // in theory, the points in the queue will approximately be a circle of
    // increasing size centred on the start position. That means that the
    // maximum number of items in it is pi * size, the diameter of the largest
    // circle which can be inscribed in the map. We use 5 as an approximation
    // to pi because we are very paranoid. In my tests, the usual factor seems
    // to be about 2.5, but I expect that to increase when we are better at
    // getting land units across water.
    // We use a queue so that points closest to the start position are
    // processed first. If we use a stack, points furthest away are
    // processed first and the algorithm uses vastly more time and space.
    int queueSize = size * 5;
    // queue of points to be processed
    PG_Point toProcess[queueSize];
    // we are going to calculate the distance (in movement points) from the
    // start position to everywhere, and store the values in here.
    int distance[length];
    int count = 0;
    for (int i=0; i<length; i++)
    {
        // -1 means don't know yet
        // -2 means can't go there at all
        // 0 or more is number of movement points needed to get there
        distance[i] = -1;
        if (isBlocked(i % size, i/size, d_dest.x, d_dest.y)) distance[i] = -2;
    }
    distance[start.y*size+start.x] = 0;
    // The rule is: whenever we change the value at (x,y), put (x,y) on the queue
    // index of the next item in the queue to process
    int nextRead = 0;
    // index of the next position in the queue to store an item into.
    int nextWrite = 0;
    toProcess[nextWrite].x = start.x;
    toProcess[nextWrite].y = start.y;
    nextWrite++;
    count++;
    while (nextRead != nextWrite)
    {
        int x = toProcess[nextRead].x;
        int y = toProcess[nextRead].y;
        nextRead++;
        if (nextRead == queueSize) nextRead = 0;
        count--;
        if (nextRead == queueSize) nextRead = 0;
        int dxy = distance[y*size+x];
        for (int sx=x-1; sx<=x+1; sx++)
        {
            if (sx < 0 || sx == size) continue; // out of bounds
            for (int sy=y-1; sy<=y+1; sy++)
            {
                if (sy < 0 || sy == size) continue; // out of bounds
                int dsxy = distance[sy*size+sx];
                if (dsxy < -1) continue; // can't move there anyway
                int p = pointsToMoveTo(sx, sy);
                int newDsxy = dxy + p;
                if (dsxy == -1 || dsxy > newDsxy)
                {
                    distance[sy*size+sx] = newDsxy;
                    toProcess[nextWrite].x = sx;
                    toProcess[nextWrite].y = sy;
                    nextWrite++;
                    count++;
                    if (nextWrite == queueSize) nextWrite = 0;
                    if (nextRead == nextWrite)
                        cerr << "Queue not long enough in path finding: " << queueSize << " " << count << " " << size;
                }
            }
        }
    }
    // The distance array is now completely populated.
    // What we have to do now is find the shortest path to the destination.
    // We do that by starting at the destination and moving at each step to
    // the neighbour closest to the start.
    int destDistance = distance[dest.y * size + dest.x];
    if (destDistance < 0) return false;
    // choose the order in which we process directions so as to favour
    // straight lines over diagonals
    int diffs[][2] = { {1, 0}, {-1, 0}, {0, -1}, {0, 1}, {-1, -1}, {-1, 1}, {1, -1}, {1, 1} };
    int dd = destDistance;
    int x = dest.x;
    int y = dest.y;
    while (dd > 0)
    {
        int min = dd;
        int rx = x;
        int ry = y;
        PG_Point *p = new PG_Point;
        p->x = rx;
        p->y = ry;
        push_front(p);
        for (int i=0; i<8; i++)
        {
            int newx = x + diffs[i][0];
            int newy = y + diffs[i][1];
            if (newx < 0 || newx == size || newy < 0 || newy == size) continue;
            int newd = distance[newy*size+newx];
            if (newd >= 0 && newd < min)
            {
                rx = newx;
                ry = newy;
                min = newd;
            }
        }
        // found the best spot to go to from
        x = rx;
        y = ry;
        dd = min;
    }
    debug("...done")
    return true;
}

int Path::pointsToMoveTo(int x, int y)
{
    // A city on the next square
    if (Citylist::getInstance()->getObjectAt(x, y)) return 1;
    unsigned int maptype = GameMap::getInstance()->getTile(x, y)->getMaptileType();
    // does everything in the stack have a bonus to move onto this square?
    bool allHaveBonus = true;
    for (Stack::iterator it = d_unit->begin(); it != d_unit->end(); it++)
    {
        if ((*it)->getMoveBonus() == maptype)
            continue;
        else
        {
            allHaveBonus = false;
            break;
        }
    }
    if (allHaveBonus) return 2;
    return GameMap::getInstance()->getTile(x, y)->getMoves();
}

void Path::calculateStackMovementBonuses()
{
    d_army_bonus = d_stack_bonus = !0;
    d_has_ship = d_has_land = false;

    //check if there are ships/non-water units in the stack and
    //calculate the move bonuses
    for (Stack::iterator it = d_unit->begin(); it != d_unit->end(); it++)
    {
        if ((*it)->getArmyBonus() & ARMY_IS_LOADED)
            continue;

        d_army_bonus &= (*it)->getArmyBonus();
        d_stack_bonus &= (*it)->getMoveBonus();

        if (((*it)->getArmyBonus()) & ARMY_SHIP)
        {
            //ship in stack
            if (!d_has_ship) debug("ship in stack")
            d_has_ship = true;
            continue;
        }

        if (!((*it)->getMoveBonus() & MAPTILE_WATER))
        {
            //unit which can't cross water
            if (!d_has_land) debug("land unit in stack")
            d_has_land = true;
            continue;
        }
    }
}

bool Path::isBlocked(int x, int y, int destx, int desty)
{
    //TODO: you can extract quite some amount of time here with a clever
    //search algorithm for stacklist

    //return true on every condition which may prevent the stack from
    //entering the tile, which are...

    //...land if the stack has ships in it...
    if (d_has_ship && (GameMap::getInstance()->getTile(x,y)->getMaptileType() != MAPTILE_WATER)
        && (GameMap::getInstance()->getTile(x,y)->getBuilding() != Maptile::CITY))
        return true;

    //...water if some units can't fly or swim...
    if (d_has_land && (GameMap::getInstance()->getTile(x,y)->getMaptileType() == MAPTILE_WATER))
        return true;

    if (Stack* s = Stacklist::getObjectAt(x,y))
    {
        //...enemy stacks which stand in the way...
        if ((s->getPlayer() != d_unit->getPlayer())
            && ((x != destx) || (y != desty)))
            return true;
        //...friendly stacks which are too big to merge with...
        if ((s->getPlayer() == d_unit->getPlayer())
            && (s->size() + d_unit->size() > 8))
            return true;
    }

    City* c = Citylist::getInstance()->getObjectAt(x,y);
    //...enemy cities
    if (c && (c->getPlayer() != d_unit->getPlayer())
        && ((x != destx) || (y != desty)))
        return true;

    //no obstacles??? well, then...
    return false;
}

Path::iterator Path::fl_erase(Path::iterator it)
{
    delete (*it);
    return erase (it);
}

void Path::fl_clear()
{
    for (Path::iterator it = begin(); it != end(); it++)
    {
        delete (*it);
    }
    return clear();
}

bool Path::save(XML_Helper* helper)
{
    bool retval = true;

    std::stringstream sx, sy;
    for (iterator it = begin(); it != end(); it++)
    {
        sx <<(*it)->x <<" ";
        sy <<(*it)->y <<" ";
    }

    retval &= helper->openTag("path");
    retval &= helper->saveData("size", size());
    retval &= helper->saveData("x", sx.str());
    retval &= helper->saveData("y", sy.str());
    retval &= helper->closeTag();

    return retval;
}
