//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef PATH_H
#define PATH_H

#include <set>
#include <map>
#include <list>
#include <paragui.h>
#include "xmlhelper.h"

class Stack;

/** The path class cares for path storing and calculation.
  *
  * The task of the path class is three-fold.
  * - it stores existing movement paths of stacks (each stack has a path
  *   instance)
  * - it checks existing paths if they are blocked (when a stack doesn't reach
  *   its target in one round, you have to validate the path in the next round)
  *   and may recalculate the path
  * - it calculates the shortest path between points on the map
  */
class Path : public std::list<PG_Point*>
{
    public:
        //! Default constructor
        Path();

        //! Loads the existing path from a savegame
        Path(XML_Helper* helper);
        ~Path();

        /** Calculates the path from the stack's location to a destination.
          *
          * The calculated path is stored within the instance (remember: each
          * stack has a path instance). During calculation, bonuses and other
          * specialities are taken into account.
          *
          * @param s            the stack whose path we calculate
          * @param _dest        the destination of the calculation
          * @return true if calculation suceeded or false if path was blocked
          */
        bool calculate(Stack* s, PG_Point dest);

        /** Validates an existing path
          *
          * @param s            the Stack whose path we validate
          * @param recalculate  if set to true, try to move around obstacles
          * @return true if path is valid, false if path is blocked (and could
          * not be recalculated)
          */
        bool checkPath(Stack* s);

        /** This function erases an item in the path list. We can't use STL
          * functions, because we need to delete the pointer as well.
          * Otherwise, it behaves like a normal erase function from the STL.
          */
        Path::iterator fl_erase(Path::iterator it);

        /** The same problem. Erases all items in the path list with the help of
          * fl_erase.
          */
        void fl_clear();

        /* This fuction is used to verify if the stack can move on the Tile
	 * of the given destination point
	 */
        bool Path::canMoveThere(Stack* s, PG_Point *_dest);

        //! Save the current path
        bool save(XML_Helper* helper);

    private:
        //the rest is purely for calculating the path
        bool isBlocked(int x, int y, int destx, int desty);

        PG_Point d_dest;
        Stack* d_unit;    //need this for is_blocked()
        unsigned int d_stack_bonus, d_army_bonus;
        bool d_has_ship;
        bool d_has_land; //has_land refers to units who can't cross water
        int pointsToMoveTo(int x, int y);
        void calculateStackMovementBonuses();
};

#endif // PATH_H
